"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("poom-utils/lib/Utils");
const HttpError_1 = require("poom-utils/lib/HttpError");
const poom_redis_1 = require("poom-redis");
const poom_mongo_1 = require("poom-mongo");
const Cached_1 = require("./Cached");
let config = {
    redisName: 'AUTHORIZ',
    timeout: 2000,
    cachedTimeOut: 5000
};
let caches;
function OAUTH_CONFIG(opts) {
    return function (_target, _fieldName, _func) {
        Object.assign(config, opts);
        caches = new Cached_1.Cached(opts.cachedTimeOut);
        caches.start();
    };
}
exports.OAUTH_CONFIG = OAUTH_CONFIG;
async function getUserByToken(token) {
    const usr = await poom_redis_1.Redis.pool(config.redisName).sendCommand(['HGET', `$tk:${token}`, 'info']);
    return (usr ? poom_mongo_1.Mongo.parse(usr) : null);
}
exports.getUserByToken = getUserByToken;
async function authen(token) {
    !token && HttpError_1.default.AUTHEN('Unauthorized', 'Lacks valid authentication credentials').throw();
    let cached = caches.get(token);
    if (cached)
        return cached;
    cached = await getUserByToken(token) || HttpError_1.default.EXPIRED('Session expired', 'Session expired').throw();
    caches.set(token, cached);
    return cached;
}
exports.authen = authen;
async function authoriz(token, path, action) {
    const cached = await authen(token);
    if (!path && !action) {
        return {
            action: '',
            account_id: cached._id,
            project_id: cached.project_id
        };
    }
    const rsCached = await poom_redis_1.Redis.pool(config.redisName).sendCommand(['HGET', `$tk:${token}`, `${action}>${path}`]);
    if (rsCached)
        return JSON.parse(rsCached);
    const rs = await _authoriz(token, path, action, cached);
    return rs;
}
exports.authoriz = authoriz;
function AUTHEN() {
    return function (target, fieldName, func) {
        const baseFunction = Utils_1.pickFunction(target, fieldName, func.value);
        func.value = function (ctx, ...args) {
            const [token, expires] = ctx.headers.token.split('?');
            return Utils_1.asyncExecute({
                func: () => authen(token),
                success: (headers) => {
                    ctx.state.auth = {
                        token,
                        expires: +expires
                    };
                    Object.defineProperties(ctx.state.auth, {
                        projectId: {
                            get() {
                                return this._projectId || (this._projectId = poom_mongo_1.Mongo.uuid(headers.project_id));
                            }
                        },
                        accountId: {
                            get() {
                                return this._accountId || (this._accountId = poom_mongo_1.Mongo.uuid(headers._id));
                            }
                        }
                    });
                    return baseFunction(ctx, ...args);
                }
            });
        };
    };
}
exports.AUTHEN = AUTHEN;
function AUTHORIZ(path = '', action = '') {
    return function (target, fieldName, func) {
        const baseFunction = Utils_1.pickFunction(target, fieldName, func.value);
        func.value = function (ctx, ...args) {
            const [token, expires] = ctx.headers.token.split('?');
            !token && HttpError_1.default.AUTHEN('Unauthorized', 'Lacks valid authentication credentials').throw();
            return Utils_1.asyncExecute({
                func: () => authoriz(token, path, action),
                success: (res) => {
                    ctx.state.auth = {
                        token,
                        expires: +expires
                    };
                    Object.defineProperties(ctx.state.auth, {
                        projectId: {
                            get() {
                                return this._projectId || (this._projectId = poom_mongo_1.Mongo.uuid(res.project_id));
                            }
                        },
                        accountId: {
                            get() {
                                return this._accountId || (this._accountId = poom_mongo_1.Mongo.uuid(res.account_id));
                            }
                        },
                        action: {
                            get() {
                                return res.action.split('&');
                            }
                        }
                    });
                    return baseFunction(ctx, ...args);
                },
                error: (err) => {
                    throw err;
                }
            });
        };
    };
}
exports.AUTHORIZ = AUTHORIZ;
async function _authoriz(token, path, action, cached) {
    const actions = action.split('|');
    const data = await Promise.race([new Promise((resolve, _reject) => {
            setTimeout(resolve, config.timeout);
        }), ...cached.api.map((r) => {
            return new Promise(resolve => {
                if ((!r._isPathRegex && r.path === path) || (r._isPathRegex && new RegExp(`^${r.path}$`).test(path))) {
                    const data = {
                        action: '',
                        account_id: cached._id,
                        project_id: cached.project_id
                    };
                    if (action) {
                        const a = !r._isActionRegex ? actions.filter(a => a === r.actions) : actions.filter(a => new RegExp(`^${r.actions}$`).test(a));
                        if (a.length > 0)
                            data.action = a.join('&');
                    }
                    poom_redis_1.Redis.pool(config.redisName).sendCommand(['HSET', `$tk:${token}`, `${action}>${path}`, JSON.stringify(data)]).then(() => {
                        resolve(data);
                    });
                }
            });
        })]) || HttpError_1.default.AUTHORIZ('Forbidden', 'You need special permission to access this API').throw();
    return data;
}
